#include <cstdio>
#include <cctype>
#include <csignal>
#include "cpluslib/string.h"

using namespace std;            // some compilers choke on std::toupper etc.

/** Compare two strings, ignoring case.
    \return 0 if equal, negative if a precedes b, positive if a follows b */
int
strCaseCmp(const char* a, const char* b)
{
    register unsigned char ac, bc;
    while ((ac = toupper((unsigned char)(*a))) == (bc = toupper((unsigned char)(*b))) && ac)
        ++a, ++b;
    return ac - bc;
}

/** Trim whitespace. Removes all whitespace from the beginning
    and end of s. */
string_t
strTrim(string_t s)
{
    return strLTrim(strRTrim(s));
}

/** Trim leading (left) whitespace. Removes all whitespace
    (see #isspace) from the beginning of s. */
string_t
strLTrim(string_t s)
{
    string_t::size_type n = 0;
    while (n < s.length() && isspace(s[n]))
        ++n;
    return s.substr(n);
}

/** Trim trailing (right) whitespace. Removes all whitespace
    (see #isspace) from the end of s. */
string_t
strRTrim(string_t s)
{
    string_t::size_type n = s.length();
    while (n > 0 && isspace(s[n-1]))
        --n;
    return s.substr(0, n);
}

/** Convert string to lower case. */
string_t
strLCase(string_t s)
{
    for (string_t::size_type n = 0; n < s.length(); ++n)
        s[n] = tolower((unsigned char) s[n]);
    return s;
}

/** Convert string to upper case. */
string_t
strUCase(string_t s)
{
    for (string_t::size_type n = 0; n < s.length(); ++n)
        s[n] = toupper((unsigned char) s[n]);
    return s;
}

/** Capitalize string. Converts all letters following a space to upper
    case, and others to lower case. */
string_t
strUCFirst(string_t s)
{
    bool uc = true;
    for (string_t::size_type n = 0; n < s.length(); ++n) {
        if (isspace(s[n])) {
            uc = true;
        } else {
            if (uc)
                s[n] = toupper((unsigned char) s[n]);
            else
                s[n] = tolower((unsigned char) s[n]);
            uc = false;
        }
    }
    return s;
}

/** Convert integer to string. */
string_t
itoa(long l)
{
    char buffer[100];
    std::sprintf(buffer, "%ld", l);
    return string_t(buffer);
}

/** Convert integer to string, adding a unit when appropriate.
    \overload */
string_t
itoa(long l, const char* a, const char* b)
{
    if (l % 101 == 1)
        return itoa(l) + a;
    else
        return itoa(l) + b;
}

/** Convert integer to string, adding a unit when appropriate.
    \param a   singular unit ("ton")
    \param b   plural unit ("tons") */
string_t
itoa(long l, const string_t& a, const string_t& b)
{
    if (l % 101 == 1)
        return itoa(l) + a;
    else
        return itoa(l) + b;
}

/** Indent string with prefix. Prepend \c prefix to all lines in \c str.
    The first line itself is not indented. */
string_t
strIndent(const string_t& prefix, string_t str)
{
    string_t::size_type p = string_t::npos;
    while (1) {
        p = str.rfind('\n', p);
        if (p == string_t::npos)
            return str;
        str.insert(p+1, prefix);
        if (!p)
            return str;
        --p;
    }
}

/** Return leading text up to separator. If there is no separator,
    return whole string. This can be used for splitting strings into
    pieces:
    \code
      string_t str = ...
      do {
         process(strFirst(str, ":"));
      } while(strRemove(std, ":"));
    \endcode */
string_t
strFirst(const string_t& str, const string_t& sep)
{
    string_t::size_type n = str.find(sep);
    if (n == string_t::npos)
        return str;
    else
        return str.substr(0, n);
}

/** Remove leading text up to separator.
    Returns true iff ok, false if the string does not contain a separator.
    \see strFirst */
bool
strRemove(string_t& str, const string_t& sep)
{
    string_t::size_type n = str.find(sep);
    if (n == string_t::npos) {
        return false;
    } else {
        str.erase(0, n + sep.length());
        return true;
    }
}

bool
strSplit(const string_t& str, string_t& lhs, string_t& rhs, const string_t& sep)
{
    string_t::size_type n = str.find(sep);
    if (n == string_t::npos) {
        return false;
    } else {
        if (&str == &lhs) {
            rhs = str.substr(n+sep.length());
            lhs.erase(n);
        } else {
            lhs = str.substr(0, n);
            rhs = str.substr(n+sep.length());
        }
        return true;
    }
}
